# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Filipe Goncalves the.wa.syndrome@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record
import gtk

from tagplayer.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('tagplayer')

import os, webbrowser, time
from threading import Thread
from daemon.lastfm import LastFM, LastFMException

import gtk_util

import logging
log = logging.getLogger("TagPlayer")

DEFAULT_MUSIC_DIR = os.path.join(os.environ['HOME'], 'Music')

class PreferencesTagplayerDialog(gtk.Dialog):
	__gtype_name__ = "PreferencesTagplayerDialog"
	preferences = {}
	__pw_changed = False

	def __new__(cls):
		"""Special static method that's automatically called by Python when 
		constructing a new instance of this class.
		
		Returns a fully instantiated PreferencesTagplayerDialog object.
		"""
		builder = get_builder('PreferencesTagplayerDialog')
		new_object = builder.get_object("preferences_tagplayer_dialog")
		new_object.finish_initializing(builder)
		return new_object

	def finish_initializing(self, builder):
		"""Called while initializing this instance in __new__

		finish_initalizing should be called after parsing the ui definition
		and creating a PreferencesTagplayerDialog object with it in order to
		finish initializing the start of the new PerferencesTagplayerDialog
		instance.
		
		Put your initialization code in here and leave __init__ undefined.
		"""

		# Get a reference to the builder and set up the signals.
		self.builder = builder
		self.builder.connect_signals(self)

		# Set up couchdb and the preference info.
		self._db_name = "tagplayer"
		self._database = CouchDatabase(self._db_name, create=True)
		self._preferences = None
		self._key = None

		# Set the record type and then initalize the preferences.
		self._record_type = (
			"http://wiki.ubuntu.com/Quickly/RecordTypes/Tagplayer/"
			"Preferences")
		self._preferences = self.get_preferences()
		#code for other initialization actions should be added here
		
		
		dir_box = self.builder.get_object('dir_box')
		
		log.debug('Preferences have %d music locations'%len(self._preferences['music_dir']))
		for x in self._preferences['music_dir']:
		    log.debug('\tMusic directory -> %s'%x)
		    md = gtk_util.MusicDirectory(x)
		    md.but.connect('clicked', self.remove_dir, md)
		    dir_box.add(md)

		if 'lfm_user' in self._preferences :#TODO transition code - remove
			del self._preferences['lfm_user']
			del self._preferences['lfm_pass']
		
		self.__pw_changed = False

	def get_preferences(self):
		"""Return a dict of preferences for tagplayer.

		Creates a couchdb record if necessary.
		"""
		if self._preferences == None:
			# The dialog is initializing.
			self._load_preferences()

		# If there were no saved preference, this.
		return self._preferences
	
	
	def add_dir(self, widget=None):
		#if box has children then add child to default folder.
		md = gtk_util.MusicDirectory(DEFAULT_MUSIC_DIR)
	    md.but.connect('clicked', self.remove_dir, md)
		self.builder.get_object('dir_box').pack_end(md)
	
	def remove_dir(self, widget, main_widget):
        log.debug('Remove music directory {0} in {1}'.format(widget, main_widget))
	    dir_box = self.builder.get_object('dir_box')
	    if len(dir_box.get_children()) > 1:
    		dir_box.remove(main_widget)
		
	def _load_preferences(self):
		# TODO: add preferences to the self._preferences dict default
		# preferences that will be overwritten if some are saved
		self._preferences = {"record_type": self._record_type}

		results = self._database.get_records(
			record_type=self._record_type, create_view=True)

		if len(results.rows) == 0:
			# No preferences have ever been saved, save them before returning.
			self._preferences['music_dir'] = [os.path.join(os.environ['HOME'],gettext.gettext('Music'))]
			self._key = self._database.put_record(Record(self._preferences))
		else:
			self._preferences = results.rows[0].value
			if type(self._preferences['music_dir']) is not list:#FIXME transition code - remove
				self._preferences['music_dir'] = [self._preferences['music_dir']]
			del self._preferences['_rev']
			self._key = results.rows[0].value["_id"]
	    
	    try:
	        self._preferences['session_key']
	        self.builder.get_object('auth_label').set_text('Authenticated')
    		self.builder.get_object('auth_button').set_label('Clear')
    	except:
    	    pass

	def _save_preferences(self):
		self._database.update_fields(self._key, self._preferences)
	
	def authenticate(self, widget=None):
		if 'session_key' in self._preferences:
			del self._preferences['session_key']
			self.builder.get_object('auth_label').set_text('Not authenticated')
			self.builder.get_object('auth_button').set_label('Authenticate')
		else:
			Thread(target=self.__do_authenticate).start()
			
			
	def __do_authenticate(self):
		self.builder.get_object('auth_label').set_text('Authenticating')
		token = LastFM.get_token()
		webbrowser.open(LastFM.auth_url(token))
		while True:
			try:
				time.sleep(1)
				self._preferences['session_key'] = LastFM.get_session(token)['key']
				break
			except LastFMException as lfm_err:
				log.debug(lfm_err)#This token has not been authorized
		self.builder.get_object('auth_label').set_text('Authenticated')
		self.builder.get_object('auth_button').set_label('Clear')


	def ok(self, widget, data=None):
		"""The user has elected to save the changes.

		Called before the dialog returns gtk.RESONSE_OK from run().
		"""

		# Make any updates to self._preferences here. e.g.
		#self._preferences["preference1"] = "value2"
		
		dir_box = self.builder.get_object('dir_box')
		
		self._preferences['music_dir'] = []
		for x in dir_box.get_children():
		    s = x.fcb.get_current_folder()
		    if s not in self._preferences['music_dir']:
		        self._preferences['music_dir'].append(s)

		self._save_preferences()

	def cancel(self, widget, data=None):
		"""The user has elected cancel changes.

		Called before the dialog returns gtk.RESPONSE_CANCEL for run()
		"""
		# Restore any changes to self._preferences here.
		pass

if __name__ == "__main__":
	dialog = PreferencesTagplayerDialog()
	dialog.show()
	gtk.main()
