# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import gobject, gtk
import dbus, dbus.service

from player import Player

import indicate

import logging
log = logging.getLogger("TagPlayer")

ROOT_IFACE = 'org.mpris.MediaPlayer2'
PLAYER_IFACE = ROOT_IFACE+'.Player'
IFACE_PATH = '/org/mpris/MediaPlayer2'

class MPRISWrapper(dbus.service.Object):

	def __init__(self, window):
		self.__player = Player(self)

		self.window = window
		
		bus_name = dbus.service.BusName(ROOT_IFACE+'.tagplayer', bus=dbus.SessionBus())
		
		dbus.service.Object.__init__(self, bus_name, IFACE_PATH)
		
		server = indicate.indicate_server_ref_default()
		server.set_type('music.{0}'.format(self.DesktopEntry))
		server.set_desktop_file('/usr/share/applications/{0}.desktop'.format(self.DesktopEntry))
		server.show()
	

	@dbus.service.method(dbus.PROPERTIES_IFACE, in_signature='s', out_signature='a{sv}')
	def GetAll(self, interface):
		log.debug('[MPRIS] GetAll in interface %s' % (interface,))
		props = { 'DesktopEntry': self.DesktopEntry, 'Metadata': self.Metadata, 'PlaybackStatus': self.PlaybackStatus, 'CanPlay': self.CanPlay, 'CanPause': self.CanPause, 'CanGoNext': self.CanGoNext, 'CanControl': self.CanControl }
#		print props
		return props

	@dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE, in_signature='ss', out_signature='v')
	def Get(self, interface, prop):
		log.debug('[MPRIS] Get property %s in interface %s' % (property, interface))
		return self.__getattribute__(prop)

	@dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE, in_signature='ssv')
	def Set(self, interface, property, value):
		log.debug('[MPRIS] Set %s in property %s in interface %s' % (str(value),property, interface))
		pass

	
	@dbus.service.method(ROOT_IFACE)
	def Raise(self):
		log.debug('[MPRIS] RAISE')
		self.window.present()
		pass
	
	@dbus.service.method(ROOT_IFACE)
	def Quit(self):
		log.debug('[MPRIS] QUIT')
		self.window.quit(None)
		pass
	
	@property
	def CanRaise(self):
		return True
	
	@property
	def CanQuit(self):
		return True
	
	@property
	def HasTracklist(self):
		return False
	
	@property
	def Identity(self):
		return 'TagPlayer'
	
	@property
	def DesktopEntry(self):
		return 'tagplayer'
	
	@dbus.service.method(dbus_interface=PLAYER_IFACE)
	def Next(self):
		log.debug('[MPRIS] NEXT')
		self.__player.next()
		self.player_info_update()

	@dbus.service.method(dbus_interface=PLAYER_IFACE)
	def PlayPause(self):
		log.debug('[MPRIS] PLAYPAUSE')
		if not self.__player.isplaying:
			self.__player.play()
		else:
			self.__player.pause()
		self.player_info_update()
		
	@dbus.service.method(dbus_interface=PLAYER_IFACE)
	def Play(self):
		log.debug('[MPRIS] PLAY')
		if not self.__player.isplaying:
			self.__player.play()
		self.player_info_update()
		
	@dbus.service.method(dbus_interface=PLAYER_IFACE)
	def Pause(self):
		log.debug('[MPRIS] PAUSE')
		if self.__player.isplaying:
			self.__player.pause()
		self.player_info_update()
	
	@property
	def CanPause(self):
		return True
	
	@property
	def CanPlay(self):
		return True
		
	@property
	def CanGoNext(self):
		return True
		
	@property
	def CanGoPrevious(self):
		return False
		
	@property
	def CanControl(self):
		return True
	
	@property
	def PlaybackStatus(self):
		return self.__player.isplaying and 'Playing' or 'Paused'
	
	@property
	def Metadata(self):
		try:
			log.debug('[MPRIS] METADATA')
			meta = self.__player.get_now_playing()
			result = dbus.Dictionary({"mpris:trackid": meta['id'], "xesam:artist": [meta['artist']], "xesam:title": meta['title'], 'xesam:url': 'file://'+ meta['path']},'sv')
			if 'album_art' in meta:
				result['mpris:artUrl']= meta['album_art']
			return result
		except:
			return dbus.Dictionary({},'ss')

	@property
	def Shuffle(self):
		return True

	@dbus.service.signal(dbus.PROPERTIES_IFACE, signature='sa{sv}as')
	def PropertiesChanged(self, interface_name, changed_properties, invalidated_properties):
		log.debug('[MPRIS] PROPERTIES CHANGED')
		if 'PlaybackStatus' in changed_properties:
			self.window.update_isplaying(changed_properties['PlaybackStatus'])
	
	#NON-DBUS METHODS
	def player_info_update(self):
		self.PropertiesChanged(PLAYER_IFACE, self.GetAll(PLAYER_IFACE),[])
		
	def set_playlist(self, x):
		self.__player.set_playlist(x)
	
	def prepare_lastfm(self, session_key):
		self.__player.prepare_lastfm(session_key)
