# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import gtk
import pygst

#pygst.require("0.10")

import urllib2
import gst

import json
import os
import time
import random
import pynotify

from threading import Thread, Lock
from lastfm import LastFM

import logging
log = logging.getLogger("TagPlayer")

class Player:
	
	
	def __init__(self, dbus_client):
	
		self.player = gst.element_factory_make("playbin2", "player")
		fakesink = gst.element_factory_make("fakesink", "fakesink")
		self.player.set_property("video-sink", fakesink)
		
		bus = self.player.get_bus()
		bus.add_signal_watch()
		bus.connect("message", self.__on_message)
		
		self.dbus_client = dbus_client
		self.scrobbler = Scrobbler()
		self.isplaying = False
		self.playlist = []
		self.playedlist =[]
		self.r = random.Random()
	
	def __notify(self):#icon matrix for pynotify at https://wiki.ubuntu.com/NotificationDevelopmentGuidelines
		def n():
			try:
				self.__notification
			except:
				pynotify.init("TagPlayer")
				self.__notification = pynotify.Notification(' ',' ', 'notification-audio-play')
			self.__notification.update(self.nowplaying['title'],'by '+self.nowplaying['artist'], icon='notification-audio-play')
			if 'album_art' in self.nowplaying:
				response=urllib2.urlopen(self.nowplaying['album_art'])
				loader=gtk.gdk.PixbufLoader()
				loader.write(response.read())
				loader.close()
				self.__notification.set_icon_from_pixbuf(loader.get_pixbuf())
			self.__notification.show()
		Thread(target=n).start()
		try:
			self.scrobbler.now_playing(self.nowplaying)
		except:
			pass
	
	def __on_message(self, bus, message):
		t = message.type
		if message.type == gst.MESSAGE_EOS:
			self.next()
			self.dbus_client.player_info_update()
		elif message.type == gst.MESSAGE_ERROR:
			self.player.set_state(gst.STATE_NULL)
			self.isplaying = False
			self.dbus_client.player_info_update()
			err, debug = message.parse_error()
			log.debug('Error on {0}'.format(message.parse_error()[0]))
	
	def prepare_lastfm(self, session_key):
		'''
			Sets up the Last.fm credentials for scrobbling
		'''
		self.scrobbler = Scrobbler(session_key)

	def set_playlist(self, playlist):
		'''
			Sets a new playlist. playlist argument receives a dictionary {artist,title,path, etc.} as exported by taglib module
		'''
		log.debug('setting playlist with {0} songs'.format(len(playlist)))
		if not playlist:
			raise Exception('Can\'t play empty list')
		self.playlist = playlist[:]
		self.playedlist = []
		self.__load_song()
	
	def __pop_song(self):
		if not self.playlist:
			self.playlist = self.playedlist
			self.playedlist = []
		song = self.r.choice(self.playlist)
		self.playedlist.append(song)
		self.playlist.remove(song)
		return song.copy()
		
	def __load_song(self):
		'''
			Prepares a song from the playlist for playback
		'''
		try:
			self.nowplaying
			a = self.player.query_duration(gst.FORMAT_TIME, None)[0]
			b = self.player.query_position(gst.FORMAT_PERCENT, None)[0]
			if b > 500000 or ( ( b / 1000000 ) * a > 240000000000 ) and a > 30000000000: #50% or over 4 minutes played and over 30 seconds long
				self.nowplaying['duration'] = int( b/1000000000 )
				self.scrobbler.scrobble(self.nowplaying)
		except:
			import sys
			log.debug(('»'*80)+'\n'+str(sys.exc_info())+'\n'+('«'*80))
		self.player.set_state(gst.STATE_NULL)
		self.nowplaying = self.__pop_song()
		self.player.set_property("uri","file://"+self.nowplaying['path'])
		self.nowplaying['timestamp'] = int(time.time())
		self.__notify()
		if self.isplaying:
			self.player.set_state(gst.STATE_PLAYING)
	
	def get_now_playing(self):
		'''
			Returns the metadata for the currenntly playing song
		'''
		return self.nowplaying
	
	def play(self):
		if not (self.playlist or self.playedlist):
			return
		self.player.set_state(gst.STATE_PLAYING)
		self.isplaying = True
	
	def pause(self):
		if not self.isplaying:
			return
		self.player.set_state(gst.STATE_PAUSED)
		self.isplaying = False
	
	def next(self):
		if not (self.playlist or self.playedlist):
			return
		self.__load_song()
	

class Scrobbler:#TODO
	'''
		TODO
			persistent scrobble queue for fault tolerance
	'''
	scrobble_queue_lock = Lock()
	scrobble_queue = [] #static var
	
	def __init__(self, session_key=None):
		self.session_key = session_key
		self.scrobble_queue  = self.__load_queue_from_disk()
		t = Thread(target=self.__save_queue_to_disk)
		t.daemon = True
		t.start()
	
	def __load_queue_from_disk(self):
		with self.scrobble_queue_lock:
			try:
				fp = open(os.path.join(os.environ['HOME'], '.tagplayer/scrobble_queue'), "r")
				result = json.load(fp)
				fp.close()
				return result
			except:
				return []
	
	def __save_queue_to_disk(self):
		while True:
			with self.scrobble_queue_lock:
				fp = open(os.path.join(os.environ['HOME'], '.tagplayer/scrobble_queue'), "w")
				json.dump(self.scrobble_queue, fp , sort_keys=True, indent = 2)
				fp.close()
			time.sleep(180)
			
	
	def __scrobble(self, metadata):
		if self.session_key:
			LastFM.scrobble(self.session_key, metadata)
		else:
			with self.scrobble_queue_lock:
				self.scrobble_queue.append(metadata)
	
	def scrobble(self, metadata):
		log.info('Scrobbling {0} from {1}'.format(metadata['title'], metadata['artist']))
		Thread(target=self.__scrobble, args=(metadata.copy(),)).start()
	
	def now_playing(self, metadata):
		if not self.session_key:
			return
		Thread(target = LastFM.updateNowPlaying, args=(self.session_key, metadata.copy())).start()
