#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
#import time
from lastfm import LastFM
from threading_util import ThreadPool
from threading import Thread, Lock
from builder import Builder
import time

import gettext
from gettext import gettext as _
gettext.textdomain('tagplayer')

import logging
log = logging.getLogger("TagPlayer")

class TagLib:
	'''
		Library management class.
		Public methods (API) are get_tag_list and get_playlist,
		and both are asynchronous.
	'''
	
	#allowed music extensions
	music_extensions = ['.mp3', '.ogg', '.wma', '.m4a', '.flac']
	
	#concurrency locks
	cacheLock = Lock()
	
	#data structures
	artistcache = {}
	__b = {}
	
	def __init__(self, music_path=[os.path.join(os.environ['HOME'],gettext.gettext('Music'))]):
		"""
			Class constructor
		"""
		self.__music_dir = music_path
		for x in self.__music_dir:
			if os.path.isdir(x):
				self.__b[x] = Builder(x,self)
			else:
				log.warning('Dir {} rejected'.format(x))
				self.__music_dir.remove(x)
		
		tp = ThreadPool(len(self.__music_dir))
		t = Thread(target=tp.map ,args=(lambda x: self.__b[x].build(), self.__b))
		t.daemon = True
		t.start()
		
	##API methods
	
	def get_playlist(self, tags):
		'''
			Returns tuple of (artist, title, path) of songs tagged with given arguments
		'''
		log.debug('get_playlist ({0})'.format(len(self.__b)))
		tp = ThreadPool(len(self.__music_dir))
		l=tp.map(lambda md: self.__b[md].get_playlist(tags) ,self.__music_dir)
		log.debug(str(l)+'\n'+str('~'*80))
		return reduce(lambda a,b: a+b, l, [])
	
	def get_tag_list(self):
		'''
			Returns a tuple with list of all the saved tags
			and the status of the library (whether it's completely built)
		'''
		log.debug('get_tag_list ({0})'.format(len(self.__b)))
		return (reduce(lambda x,y: x+y, map(lambda x: self.__b[x].get_taglist().keys() ,self.__music_dir), [] ), reduce(lambda a,b: a and b, map(lambda x: self.__b[x].complete, self.__b), True ))
	
	def get_weighted_tag_list(self):
		'''
			Returns a tuple with list of all the saved tags and their weights
			and the status of the library (whether it's completely built)
		'''
		log.debug('get_weighted_tag_list ({0})'.format(len(self.__b)))
		tp = ThreadPool(len(self.__music_dir))
		tl = tp.map(lambda x: self.__b[x].get_taglist(), self.__music_dir)
		result = {}
		for t in tl:
			for x in t:
				if x in result:
#					print x+' is repeated!!!'
					result[x] += t[x]['weight']
				else:
					result[x] = t[x]['weight']
		return (result, reduce(lambda a,b: a and b, map(lambda x: self.__b[x].complete, self.__b), True) )
	
	def __tag_intersect(self, tags, music_dir):
		'''
			Given an arbitrary number of tags, return songs in common with all
		'''
		return filter( lambda x: x is not None, reduce(lambda y,x: map(lambda z: z in y and z or None, x) , (self.__b[music_dir].get_taglist()[t]['track_list'] for t in filter(lambda k: k in self.__b[music_dir].get_taglist().keys(), tags)), [] ))

#RUN AS STANDALONE
if __name__ == '__main__':
	tl = TagLib()
	cmd = raw_input('> ')
	while(cmd!='exit' and cmd !='quit'):
		if(cmd == 'tags'):
			print len(tl.get_tag_list()[0])
		elif (cmd == 'status'):
			print tl.get_tag_list()[1]
		elif (cmd == 'pl'):
			print tl.get_playlist(['indie'])
		cmd = raw_input('> ')
