#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from mutagen import File
import os.path

import logging
log = logging.getLogger("TagPlayer")

class Util:

	@staticmethod
	def get_track_metadata(track_path):
		'''
			Given a song's file path, returns respective title and artist metadata 
		'''
		extension = os.path.splitext(track_path)[-1]
		log.debug('Getting {} metadata...'.format(track_path))
		audio = File(track_path, easy=True)
		log.debug('Got {}'.format(audio))
		artist = audio['artist'][0] if 'artist' in audio else ''
		title = audio['title'][0] if 'title' in audio else ''
		if not audio: log.warning("Failed to get metadata for {}".format(track_path))
		return (artist,title)
	
	@staticmethod
	def filter_gen(func, it):
		'''
			Filter function to apply to iterables.
			Contrary to common filter function returns a generator.
		'''
		for x in it:
			if func(x):
				yield x
	
	@staticmethod
	def map_to_gen(func, it):
		'''
			Map funtion to apply to iterables.
			This map function is error tolerant and returns a generator rather than a list. 
		'''
		for x in it:
			yield func(x)
	
	@staticmethod
	def flatten(iterables):
		'''
			Chain concatenates every list in a list of lists (or generator in list of generators) and returns it as a generator.
		'''
		for it in iterables:
			for element in it:
				yield element

